//********************************************************************************
//                                                                       
//  Company : CoreChange Co., Kr.
//  Written by : CoreChange
//  Updated by : youmi301@gmail.com 
//  Start Date : 2006.8
//	Update Date : 2007.8
//  description :
//
//*********************************************************************************

#include "stdafx.h"
#include "SeedEncDec.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSeedEncDec::CSeedEncDec()
{
	memset(rK, 0, sizeof(rK));
}

CSeedEncDec::~CSeedEncDec()
{
}

void CSeedEncDec::SD(BYTE *pbData, int nSize)
{
	if (nSize < SeedBlockSize) return; // min 16 bytes
	for(int off = 0; off < nSize; off += SeedBlockSize)
		SD(&pbData[off]);
}

void CSeedEncDec::SE(BYTE *pbData, int nSize)
{
	if (nSize < SeedBlockSize) return; // min 16 bytes
	for(int off = 0; off < nSize; off += SeedBlockSize)
		SE(&pbData[off]);
}

/************************ Block Encryption *************************/
void CSeedEncDec::SE(BYTE *pbData)  
{
	DWORD L0, L1, R0, R1, T0, T1, *K = rK;

    L0 = ((DWORD*)pbData)[0];
    L1 = ((DWORD*)pbData)[1];
    R0 = ((DWORD*)pbData)[2];
    R1 = ((DWORD*)pbData)[3];
#ifndef BIG_ENDIAN
    L0 = EndianChange(L0);
    L1 = EndianChange(L1);
    R0 = EndianChange(R0);
    R1 = EndianChange(R1);
#endif

    SeedRound(L0, L1, R0, R1, K  ); /*   1 */
    SeedRound(R0, R1, L0, L1, K+ 2); /*   2 */
    SeedRound(L0, L1, R0, R1, K+ 4); /*   3 */
    SeedRound(R0, R1, L0, L1, K+ 6); /*   4 */
    SeedRound(L0, L1, R0, R1, K+ 8); /*   5 */
    SeedRound(R0, R1, L0, L1, K+10); /*   6 */
    SeedRound(L0, L1, R0, R1, K+12); /*   7 */
    SeedRound(R0, R1, L0, L1, K+14); /*   8 */
    SeedRound(L0, L1, R0, R1, K+16); /*   9 */
    SeedRound(R0, R1, L0, L1, K+18); /*  10 */
    SeedRound(L0, L1, R0, R1, K+20); /*  11 */
    SeedRound(R0, R1, L0, L1, K+22); /*  12 */
#if NoRounds == 16
    SeedRound(L0, L1, R0, R1, K+24); /*  13 */
    SeedRound(R0, R1, L0, L1, K+26); /*  14 */
    SeedRound(L0, L1, R0, R1, K+28); /*  15 */
    SeedRound(R0, R1, L0, L1, K+30); /*  16 */
#endif

#ifndef BIG_ENDIAN
    L0 = EndianChange(L0);
    L1 = EndianChange(L1);
    R0 = EndianChange(R0);
    R1 = EndianChange(R1);
#endif
    ((DWORD *)pbData)[0] = R0;
    ((DWORD *)pbData)[1] = R1;
    ((DWORD *)pbData)[2] = L0;
    ((DWORD *)pbData)[3] = L1;

}

/* same as encrypt, except that round keys are applied in reverse order. */
void CSeedEncDec::SD(BYTE *pbData)
{
    DWORD L0, L1, R0, R1, T0, T1, *K = rK;

    L0 = ((DWORD *)pbData)[0];
    L1 = ((DWORD *)pbData)[1];
    R0 = ((DWORD *)pbData)[2];
    R1 = ((DWORD *)pbData)[3];
#ifndef BIG_ENDIAN
    L0 = EndianChange(L0);
    L1 = EndianChange(L1);
    R0 = EndianChange(R0);
    R1 = EndianChange(R1);
#endif

#if NoRounds == 16
    SeedRound(L0, L1, R0, R1, K+30); /*   1 */
    SeedRound(R0, R1, L0, L1, K+28); /*   2 */
    SeedRound(L0, L1, R0, R1, K+26); /*   3 */
    SeedRound(R0, R1, L0, L1, K+24); /*   4 */
#endif
    SeedRound(L0, L1, R0, R1, K+22); /*   5 */
    SeedRound(R0, R1, L0, L1, K+20); /*   6 */
    SeedRound(L0, L1, R0, R1, K+18); /*   7 */
    SeedRound(R0, R1, L0, L1, K+16); /*   8 */
    SeedRound(L0, L1, R0, R1, K+14); /*   9 */
    SeedRound(R0, R1, L0, L1, K+12); /*  10 */
    SeedRound(L0, L1, R0, R1, K+10); /*  11 */
    SeedRound(R0, R1, L0, L1, K+ 8); /*  12 */
    SeedRound(L0, L1, R0, R1, K+ 6); /*  13 */
    SeedRound(R0, R1, L0, L1, K+ 4); /*  14 */
    SeedRound(L0, L1, R0, R1, K+ 2); /*  15 */
    SeedRound(R0, R1, L0, L1, K+ 0); /*  16 */

#ifndef BIG_ENDIAN
    L0 = EndianChange(L0);
    L1 = EndianChange(L1);
    R0 = EndianChange(R0);
    R1 = EndianChange(R1);
#endif
    ((DWORD *)pbData)[0] = R0;
    ((DWORD *)pbData)[1] = R1;
    ((DWORD *)pbData)[2] = L0;
    ((DWORD *)pbData)[3] = L1;
}

/* Encryption key schedule */
void CSeedEncDec::SeedRoundKey(BYTE *pbUserKey)
{	
	DWORD A, B, C, D, T0, T1, *K = rK;
	
	A = ((DWORD *)pbUserKey)[0];
	B = ((DWORD *)pbUserKey)[1];
	C = ((DWORD *)pbUserKey)[2];
	D = ((DWORD *)pbUserKey)[3];
#ifndef BIG_ENDIAN
	A = EndianChange(A);
	B = EndianChange(B);
	C = EndianChange(C);
	D = EndianChange(D);
#endif
  
    T0 = A + C - KC0;
    T1 = B - D + KC0;
    K[0] = SS0[GetB0(T0)] ^ SS1[GetB1(T0)] ^
           SS2[GetB2(T0)] ^ SS3[GetB3(T0)];
    K[1] = SS0[GetB0(T1)] ^ SS1[GetB1(T1)] ^
           SS2[GetB2(T1)] ^ SS3[GetB3(T1)];

    EncRoundKeyUpdate0(K+ 2, A, B, C, D, KC1);
    EncRoundKeyUpdate1(K+ 4, A, B, C, D, KC2);
    EncRoundKeyUpdate0(K+ 6, A, B, C, D, KC3);
    EncRoundKeyUpdate1(K+ 8, A, B, C, D, KC4);
    EncRoundKeyUpdate0(K+10, A, B, C, D, KC5);
    EncRoundKeyUpdate1(K+12, A, B, C, D, KC6);
    EncRoundKeyUpdate0(K+14, A, B, C, D, KC7);
    EncRoundKeyUpdate1(K+16, A, B, C, D, KC8);
    EncRoundKeyUpdate0(K+18, A, B, C, D, KC9);
    EncRoundKeyUpdate1(K+20, A, B, C, D, KC10);
    EncRoundKeyUpdate0(K+22, A, B, C, D, KC11);
#if NoRounds == 16
    EncRoundKeyUpdate1(K+24, A, B, C, D, KC12);
    EncRoundKeyUpdate0(K+26, A, B, C, D, KC13);
    EncRoundKeyUpdate1(K+28, A, B, C, D, KC14);
    EncRoundKeyUpdate0(K+30, A, B, C, D, KC15);
#endif
}

void CSeedEncDec::MFK(char *strFilekey)
{
	char pbUserKey[16]; 
	int passwdlen = (int)strlen(strFilekey);
	memcpy(pbUserKey, strFilekey, passwdlen);
	for (int j = passwdlen; j < 16; j++) 
		pbUserKey[j] = (char)0x80;
	SeedRoundKey((BYTE *) pbUserKey);
}
